<?php

namespace Fufle\ORM;

use Fufle\ORM\Database\ConnectionManager;

abstract class Model {
    protected static string $primaryKey = 'id';
    protected static ?string $table = null;
    protected static ?IQuery $query = null;
    private static array $queryModels = [];

    protected static array $fields = [];

    public function toArray(): array {
        $array = [];
        $fields = static::$fields;
        foreach($fields as $field) {
            if (isset($this->{$field})) {
                $array[$field] = $this->{$field};
            }
        }
        return $array;
    }

    public static function Instance(object $object): static {
        $obj = new static;
        $params = get_object_vars($object);
        foreach ($params as $key => $value) {
            $obj->{$key} = $value;
        }
        return $obj;
    }

    protected static function QueryModel() {
        if (!isset(static::$queryModels[static::class]))
        if (static::$query === null) {
            static::$queryModels[static::class] = static::SetQueryModel();
        }
        return static::$queryModels[static::class];
    }

    public static function GetTableName() {
        if (static::$table === null) {
            static::$table = basename(str_replace('\\', '/', get_called_class())) . 's';
        }
        return static::$table;
    }

    public static function GetPrimaryKey() {
        return static::$primaryKey;
    }

    public static function Get(int $id) {
        return self::Query(
            static::QueryModel()->Get(),
            [$id]);
    }

    protected static function Query(string $sql, array|object $params = []) {
        $result = ConnectionManager::Query(
            $sql,
            $params,
            get_called_class());
        if (!$result) {
            return null;
        }
        return $result;
    }

    protected static function QueryAll(string $sql, array|object $params = []) {
        $result = ConnectionManager::QueryAll(
            $sql,
            $params,
            get_called_class());
        if (!$result) {
            return [];
        }
        return $result;
    }

    abstract public static function SetQueryModel(): IQuery;
}